package example3;

import java.util.Collection;
import java.util.Iterator;

import com.logntw.eva.cdm.collection.AbstractCollectionImp;
import com.logntw.eva.cdm.element.EntityIterator;
import com.logntw.eva.orm.CollectionMapper;

public class MySetImp<E> extends AbstractCollectionImp<MySet<E>, MySetEntity<E>> implements MySet<E>
{
	private final MySet<E>							set;
	private final CollectionMapper<MySetEntity<E>>	mapper;

	public MySetImp(CollectionMapper<MySetEntity<E>> mapper, Object owner, MySet<E> set)
	{
		super(owner);
		if (mapper == null || set == null)
			throw new java.lang.IllegalArgumentException();
		this.mapper = mapper;
		this.set = set;
	}

	@Override
	public Object collection()
	{
		return set;
	}

	@Override
	public CollectionMapper mapper()
	{
		return mapper;
	}

	@Override
	public MySetEntity<E> entity(Object member)
	{
		MySetEntity<E> entity = mapper.newObject();
		entity.setVal((E) member);
		return entity;
	}

	@Override
	public boolean add(E e)
	{
		boolean rtn = set.add(e);
		if (rtn)
		{
			MySetEntity<E> entity = mapper.newObject();
			entity.setVal(e);
			return mapper.insert(this, entity);
		}
		return rtn;
	}

	private static final class MySetEntityIterator<E> extends EntityIterator<E, MySetEntity<E>>
	{
		private final MySetEntity<E>	entity;

		public MySetEntityIterator(Iterable<? extends E> iterable, MySetEntity<E> entity)
		{
			super(iterable);
			this.entity = entity;
		}

		@Override
		protected MySetEntity<E> toEntity(E e)
		{
			entity.setVal(e);
			return entity;
		}
	}

	@Override
	public boolean addAll(Collection<? extends E> c)
	{
		boolean rtn = set.addAll(c);
		if (rtn)
		{
			Iterable<MySetEntity<E>> iterable = new MySetEntityIterator<E>(c, mapper.newObject());
			return allTrue(mapper.insert(this, iterable));
		}
		return rtn;
	}

	@Override
	public void clear()
	{
		set.clear();
		mapper.deleteAll(this);
	}

	@Override
	public boolean contains(Object o)
	{
		return set.contains(o);
	}

	@Override
	public boolean containsAll(Collection<?> c)
	{
		return set.containsAll(c);
	}

	@Override
	public boolean isEmpty()
	{
		return set.isEmpty();
	}

	@Override
	public Iterator<E> iterator()
	{
		return set.iterator();
	}

	@Override
	public boolean remove(Object o)
	{
		boolean rtn = set.remove(o);
		if (rtn)
		{
			MySetEntity<E> entity = mapper.newObject();
			entity.setVal((E) o);
			return mapper.delete(this, true, entity);
		}
		return rtn;
	}

	@Override
	public boolean removeAll(Collection<?> c)
	{
		boolean rtn = set.removeAll(c);
		if (rtn)
		{
			Iterable<MySetEntity<E>> iterable = new MySetEntityIterator<E>((Collection<E>) c, mapper.newObject());
			return allTrue(mapper.delete(this, true, iterable));
		}
		return rtn;
	}

	@Override
	public boolean retainAll(Collection<?> c)
	{
		throw new java.lang.UnsupportedOperationException();
	}

	@Override
	public int size()
	{
		return set.size();
	}

	@Override
	public Object[] toArray()
	{
		return set.toArray();
	}

	@Override
	public <T> T[] toArray(T[] a)
	{
		return set.toArray(a);
	}
	
	@Override
	public void eagerToGetCollection()
	{
		// do nothing.	
	}
		
	@Override
	public boolean hasCollectionElement()
	{
		return false;
	}

	@Override
	public String toString()
	{
		return set.toString();
	}
}
