package example3;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.List;

import com.logntw.eva.bean.condition.ConditionObj;
import com.logntw.eva.bean.condition.ConditionObjUtil;
import com.logntw.eva.condition.OP;
import com.logntw.eva.orm.DBConnectionProvider;
import com.logntw.eva.orm.EagerToGet;
import com.logntw.eva.orm.ORMapper;
import com.logntw.eva.orm.ORMappers;
import com.logntw.eva.orm.TxReentrantConnection;
import com.logntw.eva.orm.defo.DefaultTableCreator;
import com.logntw.eva.orm.defo.DefoORMappers;
import com.logntw.eva.orm.defo.conn.HSQLDB_1_8ConnectionProvider;

public final class QueryExample
{
	private QueryExample()
	{

	}

	public static void example1() throws Exception
	{
		DBConnectionProvider db = new HSQLDB_1_8ConnectionProvider("dbName");
		ORMappers mappers = DefoORMappers.getInstance(db);
		DefaultTableCreator.create(mappers, EntityN.class);
		DefaultTableCreator.create(mappers, EntityM.class);
		DefaultTableCreator.create(mappers, EntityL.class);

		mappers.open();
		try
		{
			ConditionObj<EntityL> cL = mappers.newCondition(EntityL.class);
			ConditionObj<EntityM> cM1 = mappers.newCondition(EntityM.class);
			ConditionObj<EntityM> cM2 = mappers.newCondition(EntityM.class);
			ConditionObj<EntityN> cN1 = mappers.newCondition(EntityN.class);
			ConditionObj<EntityN> cN2 = mappers.newCondition(EntityN.class);

			cN1.If(OP.LESS).setId6(10);
			cN2.If(OP.LIKE).setData7("A%");
			cM1.If(OP.THAT).setData5(cN1.This());
			cM2.If(OP.THAT).setData5(cN2.This());
			cL.If(OP.THAT).setData2(cM1.This());
			cL.If(OP.THAT).setData3(cM2.This());

			List<EntityL> list = mappers.select(true, EagerToGet.NONE, cL);
		}
		finally
		{
			mappers.close();
			DefaultTableCreator.drop(mappers, EntityL.class);
			DefaultTableCreator.drop(mappers, EntityM.class);
			DefaultTableCreator.drop(mappers, EntityN.class);
		}
	}

	public static void example2() throws Exception
	{
		DBConnectionProvider db = new HSQLDB_1_8ConnectionProvider("dbName");
		ORMappers mappers = DefoORMappers.getInstance(db);
		DefaultTableCreator.create(mappers, EntityN.class);
		DefaultTableCreator.create(mappers, EntityM.class);
		DefaultTableCreator.create(mappers, EntityL.class);

		mappers.open();
		try
		{
			ConditionObj<EntityL> cL = mappers.newCondition(EntityL.class);

			String conditionStr = "data2.data5.id6 < ? and data3.data5.data7 like ?";
			ConditionObjUtil.setCondition(cL, conditionStr, 10, "A%");

			List<EntityL> list = mappers.select(true, EagerToGet.NONE, cL);
		}
		finally
		{
			mappers.close();
			DefaultTableCreator.drop(mappers, EntityL.class);
			DefaultTableCreator.drop(mappers, EntityM.class);
			DefaultTableCreator.drop(mappers, EntityN.class);
		}
	}

	public static void example3() throws Exception
	{
		DBConnectionProvider db = new HSQLDB_1_8ConnectionProvider("dbName");
		ORMappers mappers = DefoORMappers.getInstance(db);
		DefaultTableCreator.create(mappers, EntityN.class);
		DefaultTableCreator.create(mappers, EntityM.class);
		DefaultTableCreator.create(mappers, EntityL.class);

		ORMapper<EntityL> mapper = mappers.getMapper(EntityL.class);
		mappers.open();
		try
		{
			String str = " data2.data5.id6 < ? and lower(data3.data5.data7) like ?";

			List<EntityL> list = mapper.select(EagerToGet.NONE, str, 10, "a%");
		}
		finally
		{
			mappers.close();
			DefaultTableCreator.drop(mappers, EntityL.class);
			DefaultTableCreator.drop(mappers, EntityM.class);
			DefaultTableCreator.drop(mappers, EntityN.class);
		}
	}

	public static void example4() throws Exception
	{
		DBConnectionProvider db = new HSQLDB_1_8ConnectionProvider("dbName");
		ORMappers mappers = DefoORMappers.getInstance(db);
		DefaultTableCreator.create(mappers, EntityN.class);
		DefaultTableCreator.create(mappers, EntityM.class);
		DefaultTableCreator.create(mappers, EntityL.class);

		ORMapper<EntityL> mapper = mappers.getMapper(EntityL.class);
		TxReentrantConnection conn = mappers.open();
		try
		{
			String evaStmt = "Select id1 from EntityL where data2.data5.id6 < ? and lower(data3.data5.data7) like ?";
			String sqlStmt = mapper.translate(true, evaStmt);
			PreparedStatement pstmt = conn.prepareStatement(sqlStmt);
			Object value1 = mapper.columnValue("data2.data5.id6", 10);
			pstmt.setInt(1, (Integer) value1);
			Object value2 = mapper.columnValue("data3.data5.data7", "a%");
			pstmt.setString(2, (String) value2);

			ResultSet reuslt = pstmt.executeQuery();
		}
		finally
		{
			mappers.close();
			DefaultTableCreator.drop(mappers, EntityL.class);
			DefaultTableCreator.drop(mappers, EntityM.class);
			DefaultTableCreator.drop(mappers, EntityN.class);
		}
	}

	public static void example5() throws Exception
	{
		DBConnectionProvider db = new HSQLDB_1_8ConnectionProvider("dbName");
		ORMappers mappers = DefoORMappers.getInstance(db);
		DefaultTableCreator.create(mappers, EntityN.class);
		DefaultTableCreator.create(mappers, EntityM.class);
		DefaultTableCreator.create(mappers, EntityL.class);

		ORMapper<EntityL> mapper = mappers.getMapper(EntityL.class);
		TxReentrantConnection conn = mappers.open();
		try
		{
			StringBuilder sqlStmt = new StringBuilder();
			sqlStmt.append("Select ");
			sqlStmt.append(mapper.columnName("id1"));
			sqlStmt.append(" from ");
			sqlStmt.append(mapper.tableName());
			String join = mapper.joinClause(true, "data2.data5.id6, data3.data5.data7");
			sqlStmt.append(join);
			sqlStmt.append(" where ");
			String name1 = mapper.columnName("data2.data5.id6");
			sqlStmt.append(name1).append(" < ? ");
			sqlStmt.append(" and ");
			sqlStmt.append(" lower(");
			String name2 = mapper.columnName("data3.data5.data7");
			sqlStmt.append(name2).append(") like ? ");

			PreparedStatement pstmt = conn.prepareStatement(sqlStmt.toString());
			Object value1 = mapper.columnValue("data2.data5.id6", 10);
			pstmt.setInt(1, (Integer) value1);
			Object value2 = (String) mapper.columnValue("data3.data5.data7", "a%");
			pstmt.setString(2, (String) value2);

			ResultSet reuslt = pstmt.executeQuery();
		}
		finally
		{
			mappers.close();
			DefaultTableCreator.drop(mappers, EntityL.class);
			DefaultTableCreator.drop(mappers, EntityM.class);
			DefaultTableCreator.drop(mappers, EntityN.class);
		}
	}
	
	public static void example6() throws Exception
	{
		DBConnectionProvider db = new HSQLDB_1_8ConnectionProvider("dbName");
		ORMappers mappers = DefoORMappers.getInstance(db);
		DefaultTableCreator.create(mappers, EntityN.class);
		DefaultTableCreator.create(mappers, EntityM.class);
		DefaultTableCreator.create(mappers, EntityL.class);

		ORMapper<EntityL> mapper = mappers.getMapper(EntityL.class);
		mappers.open();
		try
		{
			StringBuilder sqlStmt = new StringBuilder();
			sqlStmt.append("Select * from ");
			sqlStmt.append(mapper.tableName());
			String join = mapper.joinClause(true, "data2.data5.id6, data3.data5.data7");
			sqlStmt.append(join);
			sqlStmt.append(" where ");
			String name1 = mapper.columnName("data2.data5.id6");
			sqlStmt.append(name1).append(" < ? ");
			sqlStmt.append(" and ");
			sqlStmt.append(" lower(");
			String name2 = mapper.columnName("data3.data5.data7");
			sqlStmt.append(name2).append(") like ? ");
			
			Object value1 = mapper.columnValue("data2.data5.id6", 10);			
			Object value2 = (String) mapper.columnValue("data3.data5.data7", "a%");			

			List<EntityL> reuslt = mapper.query(sqlStmt.toString(), value1, value2); 
		}
		finally
		{
			mappers.close();
			DefaultTableCreator.drop(mappers, EntityL.class);
			DefaultTableCreator.drop(mappers, EntityM.class);
			DefaultTableCreator.drop(mappers, EntityN.class);
		}
	}
}