package example5;

import java.lang.reflect.Method;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.List;

import com.logntw.eva.anno.ioc.aop.Classes;
import com.logntw.eva.anno.ioc.aop.Methods;
import com.logntw.eva.anno.ioc.aop.WatchMethods;
import com.logntw.eva.ioc.aop.Advice;
import com.logntw.eva.ioc.aop.ObjectMonitor;
import com.logntw.eva.ioc.aop.When;
import com.logntw.eva.ioc.aop.Advice.Reaction;
import com.logntw.eva.orm.ORMapper;
import com.logntw.eva.orm.ORMappers;
import com.logntw.eva.sort.UnknownPropertyException;

public class ORMMonitor implements ObjectMonitor
{
	protected final ORMappers mappers;
	public ORMMonitor(ORMappers mappers)
	{
		if(mappers == null)
			throw new java.lang.IllegalArgumentException();
		this.mappers = mappers;
	}
	
	@Override
	public void onClassCannotBeMonitored(Class<?> clz)
	{
		throw new java.lang.IllegalStateException();
	}

	@Override
	public void onMethodCannotBeMonitored(Method method)
	{
		throw new java.lang.IllegalStateException();
	}

	@WatchMethods(clz = @Classes(), method = @Methods(with={QueryByCondition.class}), when = When.BEFORE)
	public void onSelect(Advice advice, Object... paras)
	{
		mappers.open();	
		try
		{
			QueryByCondition setting = advice.getMethod().getAnnotation(QueryByCondition.class);			
			ORMapper mapper = mappers.getMapper(setting.entity());
			List list = mapper.select(setting.eager(), setting.condition(), paras, setting.option(), setting.pageSize());
			advice.setReturnedValue(list);
			advice.setReaction(Reaction.RETURN_RIGHT_NOW);
		}
		finally
		{
			mappers.close();
		}
	}
	
	@WatchMethods(clz = @Classes(), method = @Methods(with={Update.class}), when = When.BEFORE)
	public void onExecUpdate(Advice advice, Object... paras)
	{
		Connection conn = mappers.open();	
		try
		{
			Update setting = advice.getMethod().getAnnotation(Update.class);			
			ORMapper mapper = mappers.getMapper(setting.entity());			
			PreparedStatement pstmt = conn.prepareStatement(mapper.translate(false, setting.stmt()));
			int len = paras.length;
			for(int i = 0 ; i < len ; i ++)
				pstmt.setObject(i + 1, paras[i]);		
			int rtn = pstmt.executeUpdate();			
			conn.commit();
			advice.setReturnedValue(rtn);
			advice.setReaction(Reaction.RETURN_RIGHT_NOW);
		}
		catch (UnknownPropertyException e)
		{
			advice.setException(e);
			advice.setReaction(Reaction.THROW_EXCEPTION_NOW);
		}
		catch (SQLException e)
		{
			advice.setException(e);
			advice.setReaction(Reaction.THROW_EXCEPTION_NOW);
		}
		finally
		{
			mappers.close();
		}
	}
}
